@@ -20,6 +20,7 @@
 #include "kio_gdrive.h"
 #include "gdrivedebug.h"
 #include "gdrivehelper.h"
+#include "gdriveurl.h"
 #include "gdriveversion.h"
 
 #include <QApplication>
@@ -58,18 +59,6 @@ class KIOPluginForMetaData : public QObject
     Q_PLUGIN_METADATA(IID "org.kde.kio.slave.gdrive" FILE "gdrive.json")
 };
 
-static QString joinSublist(const QStringList &strv, int start, int end, const QChar &joinChar)
-{
-    QString res = joinChar;
-    for (int i = start; i <= end; ++i) {
-        res += strv[i];
-        if (i < end) {
-            res += joinChar;
-        }
-    }
-    return res;
-}
-
 extern "C"
 {
     int Q_DECL_EXPORT kdemain(int argc, char **argv)
@@ -112,13 +101,13 @@ KIOGDrive::Action KIOGDrive::handleError(const KGAPI2::Job &job, const QUrl &url
             return Success;
         case KGAPI2::AuthCancelled:
         case KGAPI2::AuthError:
-            error(KIO::ERR_COULD_NOT_LOGIN, url.toDisplayString());
+            error(KIO::ERR_CANNOT_LOGIN, url.toDisplayString());
             return Fail;
         case KGAPI2::Unauthorized: {
             const AccountPtr oldAccount = job.account();
             const AccountPtr account = m_accountManager.refreshAccount(oldAccount);
             if (!account) {
-                error(KIO::ERR_COULD_NOT_LOGIN, url.toDisplayString());
+                error(KIO::ERR_CANNOT_LOGIN, url.toDisplayString());
                 return Fail;
             }
             return Restart;
@@ -165,7 +154,7 @@ KIO::UDSEntry KIOGDrive::fileToUDSEntry(const FilePtr &origFile, const QString &
         entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFREG);
         entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, file->mimeType());
         entry.insert(KIO::UDSEntry::UDS_SIZE, file->fileSize());
-        entry.insert(KIO::UDSEntry::UDS_URL, QString::fromLatin1("gdrive://%1/%2?id=%3").arg(path, origFile->title(), origFile->id()));
+        entry.insert(KIO::UDSEntry::UDS_URL, QStringLiteral("gdrive://%1/%2?id=%3").arg(path, origFile->title(), origFile->id()));
     }
 
     entry.insert(KIO::UDSEntry::UDS_CREATION_TIME, file->createdDate().toTime_t());
@@ -196,36 +185,13 @@ void KIOGDrive::openConnection()
 
 
 
-QStringList KIOGDrive::pathComponents(const QString &path) const
-{
-    return path.split(QLatin1Char('/'), QString::SkipEmptyParts);
-}
 
-QString KIOGDrive::accountFromPath(const QUrl &url) const
-{
-    const QStringList components = pathComponents(url);
-    if (components.isEmpty()) {
-        return QString();
-    }
-    return components[0];
-}
-
-bool KIOGDrive::isRoot(const QUrl &url) const
-{
-    return pathComponents(url).length() == 0;
-}
-
-
-bool KIOGDrive::isAccountRoot(const QUrl &url) const
-{
-    return pathComponents(url).length() == 1;
-}
 
 
 void KIOGDrive::createAccount()
 {
     const KGAPI2::AccountPtr account = m_accountManager.account(QString());
-    redirection(QUrl(QString::fromLatin1("gdrive:/%1").arg(account->accountName())));
+    redirection(QUrl(QStringLiteral("gdrive:/%1").arg(account->accountName())));
     finished();
 }
 
@@ -263,6 +229,9 @@ public:
         --sDepth;
     }
 
+    RecursionDepthCounter(const RecursionDepthCounter &) = delete;
+    RecursionDepthCounter& operator=(const RecursionDepthCounter &) = delete;
+
     int depth() const
     {
         return sDepth;
@@ -278,10 +247,6 @@ int RecursionDepthCounter::sDepth = 0;
 QString KIOGDrive::resolveFileIdFromPath(const QString &path, PathFlags flags)
 {
     qCDebug(GDRIVE) << Q_FUNC_INFO << path;
-    RecursionDepthCounter recursion;
-    if (recursion.depth() == 1) {
-        m_cache.dump();
-    }
 
     if (path.isEmpty()) {
         return QString();
@@ -293,19 +258,21 @@ QString KIOGDrive::resolveFileIdFromPath(const QString &path, PathFlags flags)
         return fileId;
     }
 
-    const QStringList components = pathComponents(path);
-    Q_ASSERT(!components.isEmpty());
-    if (components.size() == 1 || (components.size() == 2 && components[1] == QLatin1String("trash"))) {
+    QUrl url;
+    url.setScheme(QStringLiteral("gdrive"));
+    url.setPath(path);
+    const auto gdriveUrl = GDriveUrl(url);
+    Q_ASSERT(!gdriveUrl.isRoot());
+
+    const QStringList components = gdriveUrl.pathComponents();
+    if (gdriveUrl.isAccountRoot() || (components.size() == 2 && components[1] == QLatin1String("trash"))) {
         qCDebug(GDRIVE) << "Resolved" << path << "to \"root\"";
         return rootFolderId(components[0]);
     }
 
-    QString parentName;
-    Q_ASSERT(components.size() >= 2);
-    const QString parentPath = joinSublist(components, 0, components.size() - 2, QLatin1Char('/'));
     // Try to recursively resolve ID of parent path - either from cache, or by
     // querying Google
-    const QString parentId = resolveFileIdFromPath(parentPath, KIOGDrive::PathIsFolder);
+    const QString parentId = resolveFileIdFromPath(gdriveUrl.parentPath(), KIOGDrive::PathIsFolder);
     if (parentId.isEmpty()) {
         // We failed to resolve parent -> error
         return QString();
@@ -321,10 +288,7 @@ QString KIOGDrive::resolveFileIdFromPath(const QString &path, PathFlags flags)
     query.addQuery(FileSearchQuery::Parents, FileSearchQuery::In, parentId);
     query.addQuery(FileSearchQuery::Trashed, FileSearchQuery::Equals, components[1] == QLatin1String("trash"));
 
-    QUrl url;
-    url.setScheme(QStringLiteral("gdrive"));
-    url.setPath(path);
-    const QString accountId = accountFromPath(url);
+    const QString accountId = gdriveUrl.account();
     FileFetchJob fetchJob(query, getAccount(accountId));
     fetchJob.setFields(FileFetchJob::Id | FileFetchJob::Title | FileFetchJob::Labels);
     if (!runJob(fetchJob, url, accountId)) {
@@ -348,7 +312,8 @@ QString KIOGDrive::resolveFileIdFromPath(const QString &path, PathFlags flags)
 
 QString KIOGDrive::rootFolderId(const QString &accountId)
 {
-    if (!m_rootIds.contains(accountId)) {
+    auto it = m_rootIds.constFind(accountId);
+    if (it == m_rootIds.cend()) {
         AboutFetchJob aboutFetch(getAccount(accountId));
         QUrl url;
         if (!runJob(aboutFetch, url, accountId)) {
@@ -361,29 +326,29 @@ QString KIOGDrive::rootFolderId(const QString &accountId)
             return QString();
         }
 
-        m_rootIds.insert(accountId, about->rootFolderId());
-        return about->rootFolderId();
+        auto v = m_rootIds.insert(accountId, about->rootFolderId());
+        return *v;
     }
 
-    return m_rootIds[accountId];
+    return *it;
 }
 
 void KIOGDrive::listDir(const QUrl &url)
 {
     qCDebug(GDRIVE) << "Going to list" << url;
 
-    const QString accountId = accountFromPath(url);
+    const auto gdriveUrl = GDriveUrl(url);
+    const QString accountId = gdriveUrl.account();
     if (accountId == QLatin1String("new-account")) {
         createAccount();
         return;
     }
 
     QString folderId;
-    const QStringList components = pathComponents(url);
-    if (components.isEmpty())  {
+    if (gdriveUrl.isRoot())  {
         listAccounts();
         return;
-    } else if (components.size() == 1) {
+    } else if (gdriveUrl.isAccountRoot()) {
         folderId = rootFolderId(accountId);
     } else {
         folderId = m_cache.idForPath(url.path());
@@ -439,18 +404,20 @@ void KIOGDrive::mkdir(const QUrl &url, int permissions)
 
     qCDebug(GDRIVE) << "Creating directory" << url;
 
-    const QString accountId = accountFromPath(url);
-    const QStringList components = pathComponents(url);
+    const auto gdriveUrl = GDriveUrl(url);
+    const QString accountId = gdriveUrl.account();
+//    const QStringList components = pathComponents(url);
     QString parentId;
     // At least account and new folder name
-    if (components.size() < 2) {
+    if (gdriveUrl.isRoot() || gdriveUrl.isAccountRoot()) {
         error(KIO::ERR_DOES_NOT_EXIST, url.path());
         return;
-    } else if (components.size() == 2) {
+    }
+    const auto components = gdriveUrl.pathComponents();
+    if (components.size() == 2) {
         parentId = rootFolderId(accountId);
     } else {
-        const QString subpath = joinSublist(components, 0, components.size() - 2, QLatin1Char('/'));
-        parentId = resolveFileIdFromPath(subpath, KIOGDrive::PathIsFolder);
+        parentId = resolveFileIdFromPath(gdriveUrl.parentPath(), KIOGDrive::PathIsFolder);
     }
 
     if (parentId.isEmpty()) {
@@ -477,13 +444,14 @@ void KIOGDrive::stat(const QUrl &url)
 {
     qCDebug(GDRIVE) << "Going to stat()" << url;
 
-    const QString accountId = accountFromPath(url);
-    const QStringList components = pathComponents(url);
-    if (components.isEmpty()) {
+    const auto gdriveUrl = GDriveUrl(url);
+    const QString accountId = gdriveUrl.account();
+    if (gdriveUrl.isRoot()) {
         // TODO Can we stat() root?
         finished();
         return;
-    } else if (components.size() == 1) {
+    }
+    if (gdriveUrl.isAccountRoot()) {
         const KIO::UDSEntry entry = AccountManager::accountToUDSEntry(accountId);
         statEntry(entry);
         finished();
@@ -491,8 +459,8 @@ void KIOGDrive::stat(const QUrl &url)
     }
 
     const QString fileId
-        = url.hasQueryItem(QLatin1String("id"))
-            ? QUrlQuery(url).queryItemValue(QLatin1String("id"))
+        = url.hasQueryItem(QStringLiteral("id"))
+            ? QUrlQuery(url).queryItemValue(QStringLiteral("id"))
             : resolveFileIdFromPath(url.adjusted(QUrl::StripTrailingSlash).path(),
                                     KIOGDrive::None);
     if (fileId.isEmpty()) {
@@ -515,7 +483,7 @@ void KIOGDrive::stat(const QUrl &url)
         return;
     }
 
-    const KIO::UDSEntry entry = fileToUDSEntry(file, joinSublist(components, 0, components.size() - 2, QLatin1Char('/')));
+    const KIO::UDSEntry entry = fileToUDSEntry(file, gdriveUrl.parentPath());
 
     statEntry(entry);
     finished();
@@ -525,21 +493,22 @@ void KIOGDrive::get(const QUrl &url)
 {
     qCDebug(GDRIVE) << "Fetching content of" << url;
 
-    const QString accountId = accountFromPath(url);
-    const QStringList components = pathComponents(url);
+    const auto gdriveUrl = GDriveUrl(url);
+    const QString accountId = gdriveUrl.account();
 
-    if (components.isEmpty()) {
+    if (gdriveUrl.isRoot()) {
         error(KIO::ERR_DOES_NOT_EXIST, url.path());
         return;
-    } else if (components.size() == 1) {
+    }
+    if (gdriveUrl.isAccountRoot()) {
         // You cannot GET an account folder!
         error(KIO::ERR_ACCESS_DENIED, url.path());
         return;
     }
 
     const QString fileId =
-        url.hasQueryItem(QLatin1String("id"))
-            ? QUrlQuery(url).queryItemValue(QLatin1String("id"))
+        url.hasQueryItem(QStringLiteral("id"))
+            ? QUrlQuery(url).queryItemValue(QStringLiteral("id"))
             : resolveFileIdFromPath(url.adjusted(QUrl::StripTrailingSlash).path(),
                                     KIOGDrive::PathIsFile);
     if (fileId.isEmpty()) {
@@ -588,7 +557,7 @@ bool KIOGDrive::readPutData(QTemporaryFile &tempFile)
     // TODO: Support resumable upload (requires support in LibKGAPI)
 
     if (!tempFile.open()) {
-        error(KIO::ERR_COULD_NOT_WRITE, tempFile.fileName());
+        error(KIO::ERR_CANNOT_WRITE, tempFile.fileName());
         return false;
     }
 
@@ -600,7 +569,7 @@ bool KIOGDrive::readPutData(QTemporaryFile &tempFile)
         if (!buffer.isEmpty()) {
             qint64 size = tempFile.write(buffer);
             if (size != buffer.size()) {
-                error(KIO::ERR_COULD_NOT_WRITE, tempFile.fileName());
+                error(KIO::ERR_CANNOT_WRITE, tempFile.fileName());
                 return false;
             }
         }
@@ -609,7 +578,7 @@ bool KIOGDrive::readPutData(QTemporaryFile &tempFile)
 
     if (result == -1) {
         qCWarning(GDRIVE) << "Could not read source file" << tempFile.fileName();
-        error(KIO::ERR_COULD_NOT_READ, QString());
+        error(KIO::ERR_CANNOT_READ, QString());
         return false;
     }
 
@@ -638,11 +607,14 @@ bool KIOGDrive::runJob(KGAPI2::Job &job, const QUrl &url, const QString &account
     return true;
 }
 
-bool KIOGDrive::putUpdate(const QUrl &url, const QString &accountId, const QStringList &pathComponents)
+bool KIOGDrive::putUpdate(const QUrl &url)
 {
     const QString fileId = QUrlQuery(url).queryItemValue(QStringLiteral("id"));
     qCDebug(GDRIVE) << Q_FUNC_INFO << url << fileId;
 
+    const auto gdriveUrl = GDriveUrl(url);
+    const auto accountId = gdriveUrl.account();
+
     FileFetchJob fetchJob(fileId, getAccount(accountId));
     if (!runJob(fetchJob, url, accountId)) {
         return false;
@@ -650,14 +622,14 @@ bool KIOGDrive::putUpdate(const QUrl &url, const QString &accountId, const QStri
 
     const ObjectsList objects = fetchJob.items();
     if (objects.size() != 1) {
-        putCreate(url, accountId, pathComponents);
+        putCreate(url);
         return false;
     }
 
     const FilePtr file = objects[0].dynamicCast<File>();
     QTemporaryFile tmpFile;
     if (!readPutData(tmpFile)) {
-        error(KIO::ERR_COULD_NOT_READ, url.path());
+        error(KIO::ERR_CANNOT_READ, url.path());
         return false;
     }
 
@@ -670,17 +642,21 @@ bool KIOGDrive::putUpdate(const QUrl &url, const QString &accountId, const QStri
     return true;
 }
 
-bool KIOGDrive::putCreate(const QUrl &url, const QString &accountId, const QStringList &components)
+bool KIOGDrive::putCreate(const QUrl &url)
 {
     qCDebug(GDRIVE) << Q_FUNC_INFO << url;
     ParentReferencesList parentReferences;
-    if (components.size() < 2) {
+
+    const auto gdriveUrl = GDriveUrl(url);
+    if (gdriveUrl.isRoot() || gdriveUrl.isAccountRoot()) {
         error(KIO::ERR_ACCESS_DENIED, url.path());
         return false;
-    } else if (components.length() == 2) {
+    }
+    const auto components = gdriveUrl.pathComponents();
+    if (components.length() == 2) {
         // Creating in root directory
     } else {
-        const QString parentId = resolveFileIdFromPath(joinSublist(components, 0, components.size() - 2, QLatin1Char('/')));
+        const QString parentId = resolveFileIdFromPath(gdriveUrl.parentPath());
         if (parentId.isEmpty()) {
             error(KIO::ERR_DOES_NOT_EXIST, url.adjusted(QUrl::RemoveFilename|QUrl::StripTrailingSlash).path());
             return false;
@@ -701,10 +677,11 @@ bool KIOGDrive::putCreate(const QUrl &url, const QString &accountId, const QStri
 
     QTemporaryFile tmpFile;
     if (!readPutData(tmpFile)) {
-        error(KIO::ERR_COULD_NOT_READ, url.path());
+        error(KIO::ERR_CANNOT_READ, url.path());
         return false;
     }
 
+    const auto accountId = gdriveUrl.account();
     FileCreateJob createJob(tmpFile.fileName(), file, getAccount(accountId));
     if (!runJob(createJob, url, accountId)) {
         return false;
@@ -724,15 +701,12 @@ void KIOGDrive::put(const QUrl &url, int permissions, KIO::JobFlags flags)
 
     qCDebug(GDRIVE) << Q_FUNC_INFO << url;
 
-    const QString accountId = accountFromPath(url);
-    const QStringList components = pathComponents(url);
-
     if (url.hasQueryItem(QStringLiteral("id"))) {
-        if (!putUpdate(url, accountId, components)) {
+        if (!putUpdate(url)) {
             return;
         }
     } else {
-        if (!putCreate(url, accountId, components)) {
+        if (!putCreate(url)) {
             return;
         }
     }
@@ -757,8 +731,10 @@ void KIOGDrive::copy(const QUrl &src, const QUrl &dest, int permissions, KIO::Jo
     // name will be created.
     Q_UNUSED(flags);
 
-    const QString sourceAccountId = accountFromPath(src);
-    const QString destAccountId = accountFromPath(dest);
+    const auto srcGDriveUrl = GDriveUrl(src);
+    const auto destGDriveUrl = GDriveUrl(dest);
+    const QString sourceAccountId = srcGDriveUrl.account();
+    const QString destAccountId = destGDriveUrl.account();
 
     // TODO: Does this actually happen, or does KIO treat our account name as host?
     if (sourceAccountId != destAccountId) {
@@ -767,17 +743,17 @@ void KIOGDrive::copy(const QUrl &src, const QUrl &dest, int permissions, KIO::Jo
         return;
     }
 
-    const QStringList srcPathComps = pathComponents(src);
-    if (srcPathComps.isEmpty()) {
+    if (srcGDriveUrl.isRoot()) {
         error(KIO::ERR_DOES_NOT_EXIST, src.path());
         return;
-    } else if (srcPathComps.size() == 1) {
+    }
+    if (srcGDriveUrl.isAccountRoot()) {
         error(KIO::ERR_ACCESS_DENIED, src.path());
         return;
     }
 
     const QString sourceFileId
-        = src.hasQueryItem(QLatin1String("id"))
+        = src.hasQueryItem(QStringLiteral("id"))
               ? QUrlQuery(src).queryItemValue(QStringLiteral("id"))
               : resolveFileIdFromPath(src.adjusted(QUrl::StripTrailingSlash).path());
     if (sourceFileId.isEmpty()) {
@@ -797,12 +773,13 @@ void KIOGDrive::copy(const QUrl &src, const QUrl &dest, int permissions, KIO::Jo
 
     const FilePtr sourceFile = objects[0].dynamicCast<File>();
 
-    const QStringList destPathComps = pathComponents(dest);
     ParentReferencesList destParentReferences;
-    if (destPathComps.isEmpty()) {
+    if (destGDriveUrl.isRoot()) {
         error(KIO::ERR_ACCESS_DENIED, dest.path());
         return;
-    } else if (destPathComps.size() == 1) {
+    }
+    const auto destPathComps = destGDriveUrl.pathComponents();
+    if (destGDriveUrl.isAccountRoot()) {
         // copy to root
     } else {
         const QString destDirId = destPathComps[destPathComps.count() - 2];
@@ -840,19 +817,19 @@ void KIOGDrive::del(const QUrl &url, bool isfile)
     qCDebug(GDRIVE) << "Deleting URL" << url << "- is it a file?" << isfile;
 
     const QString fileId
-        = isfile && url.hasQueryItem(QLatin1String("id"))
-            ? QUrlQuery(url).queryItemValue(QLatin1String("id"))
+        = isfile && url.hasQueryItem(QStringLiteral("id"))
+            ? QUrlQuery(url).queryItemValue(QStringLiteral("id"))
             : resolveFileIdFromPath(url.adjusted(QUrl::StripTrailingSlash).path(),
                                     isfile ? KIOGDrive::PathIsFile : KIOGDrive::PathIsFolder);
     if (fileId.isEmpty()) {
         error(KIO::ERR_DOES_NOT_EXIST, url.path());
         return;
     }
-    const QString accountId = accountFromPath(url);
-    const QStringList components = pathComponents(url);
+    const auto gdriveUrl = GDriveUrl(url);
+    const QString accountId = gdriveUrl.account();
 
     // If user tries to delete the account folder, remove the account from the keychain
-    if (components.count() == 1) {
+    if (gdriveUrl.isAccountRoot()) {
         const KGAPI2::AccountPtr account = m_accountManager.account(accountId);
         if (!account) {
             error(KIO::ERR_DOES_NOT_EXIST, accountId);
@@ -871,8 +848,8 @@ void KIOGDrive::del(const QUrl &url, bool isfile)
         runJob(referencesFetch, url, accountId);
         const bool isEmpty = !referencesFetch.items().count();
 
-        if (!isEmpty && metaData("recurse") != QLatin1String("true")) {
-            error(KIO::ERR_COULD_NOT_RMDIR, url.path());
+        if (!isEmpty && metaData(QStringLiteral("recurse")) != QLatin1String("true")) {
+            error(KIO::ERR_CANNOT_RMDIR, url.path());
             return;
         }
     }
@@ -891,8 +868,10 @@ void KIOGDrive::rename(const QUrl &src, const QUrl &dest, KIO::JobFlags flags)
     Q_UNUSED(flags)
     qCDebug(GDRIVE) << "Renaming" << src << "to" << dest;
 
-    const QString sourceAccountId = accountFromPath(src);
-    const QString destAccountId = accountFromPath(dest);
+    const auto srcGDriveUrl = GDriveUrl(src);
+    const auto destGDriveUrl = GDriveUrl(dest);
+    const QString sourceAccountId = srcGDriveUrl.account();
+    const QString destAccountId = destGDriveUrl.account();
 
     // TODO: Does this actually happen, or does KIO treat our account name as host?
     if (sourceAccountId != destAccountId) {
@@ -900,17 +879,17 @@ void KIOGDrive::rename(const QUrl &src, const QUrl &dest, KIO::JobFlags flags)
         return;
     }
 
-    const QStringList srcPathComps = pathComponents(src);
-    if (srcPathComps.isEmpty()) {
+    if (srcGDriveUrl.isRoot()) {
         error(KIO::ERR_DOES_NOT_EXIST, dest.path());
         return;
-    } else if (srcPathComps.size() == 1) {
+    }
+    if (srcGDriveUrl.isAccountRoot()) {
         error(KIO::ERR_ACCESS_DENIED, dest.path());
         return;
     }
     const QString sourceFileId
-        = src.hasQueryItem(QLatin1String("id"))
-            ? QUrlQuery(src).queryItemValue(QLatin1String("id"))
+        = src.hasQueryItem(QStringLiteral("id"))
+            ? QUrlQuery(src).queryItemValue(QStringLiteral("id"))
             : resolveFileIdFromPath(src.adjusted(QUrl::StripTrailingSlash).path(),
                                     KIOGDrive::PathIsFile);
     if (sourceFileId.isEmpty()) {
@@ -932,19 +911,19 @@ void KIOGDrive::rename(const QUrl &src, const QUrl &dest, KIO::JobFlags flags)
     const FilePtr sourceFile = objects[0].dynamicCast<File>();
 
     ParentReferencesList parentReferences = sourceFile->parents();
-    const QStringList destPathComps = pathComponents(dest);
-    if (destPathComps.isEmpty()) {
-        // paths.size == 0 -> error, user is trying to move to top-level gdrive:///
+    if (destGDriveUrl.isRoot()) {
+        // user is trying to move to top-level gdrive:///
         error(KIO::ERR_ACCESS_DENIED, dest.fileName());
         return;
-    } else if (destPathComps.size() == 1) {
+    }
+    const auto srcPathComps = srcGDriveUrl.pathComponents();
+    const auto destPathComps = destGDriveUrl.pathComponents();
+    if (destGDriveUrl.isAccountRoot()) {
         // user is trying to move to root -> we are only renaming
     } else {
          // skip filename and extract the second-to-last component
-        const QString destDirId = resolveFileIdFromPath(joinSublist(destPathComps, 0, destPathComps.count() - 2, QLatin1Char('/')),
-                                                        KIOGDrive::PathIsFolder);
-        const QString srcDirId = resolveFileIdFromPath(joinSublist(srcPathComps, 0, srcPathComps.count() - 2, QLatin1Char('/')),
-                                                       KIOGDrive::PathIsFolder);
+        const QString destDirId = resolveFileIdFromPath(destGDriveUrl.parentPath(), KIOGDrive::PathIsFolder);
+        const QString srcDirId = resolveFileIdFromPath(srcGDriveUrl.parentPath(), KIOGDrive::PathIsFolder);
 
         // Remove source from parent references
         auto iter = parentReferences.begin();
@@ -986,14 +965,14 @@ void KIOGDrive::mimetype(const QUrl &url)
     qCDebug(GDRIVE) << Q_FUNC_INFO << url;
 
     const QString fileId
-        = url.hasQueryItem(QLatin1String("id"))
-            ? QUrlQuery(url).queryItemValue(QLatin1String("id"))
+        = url.hasQueryItem(QStringLiteral("id"))
+            ? QUrlQuery(url).queryItemValue(QStringLiteral("id"))
             : resolveFileIdFromPath(url.adjusted(QUrl::StripTrailingSlash).path());
     if (fileId.isEmpty()) {
         error(KIO::ERR_DOES_NOT_EXIST, url.path());
         return;
     }
-    const QString accountId = accountFromPath(url);
+    const QString accountId = GDriveUrl(url).account();
 
     FileFetchJob fileFetchJob(fileId, getAccount(accountId));
     fileFetchJob.setFields(FileFetchJob::Id | FileFetchJob::MimeType);
